"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const dotenv_1 = require("dotenv");
dotenv_1.config();
const server_1 = require("./server");
const config_reader_1 = require("./config.reader");
const responseTime_1 = require("./middlewares/responseTime");
const nameSpaceHandler_1 = require("./middlewares/nameSpaceHandler");
const ctx_provider_1 = require("./ctx.provider");
const compose = require("koa-compose");
const middlewares = [];
const cacheConfig = config_reader_1.getCacheConfig();
if (process.env.TIMEOUT) {
}
for (const key in cacheConfig) {
    if (!Object.prototype.hasOwnProperty.call(cacheConfig, key))
        continue;
    switch (key) {
        case 'version': break;
        case 'xResponseTime':
            middlewares.push(responseTime_1.RouteTimeReqRes);
            break;
        default:
            middlewares.push(nameSpaceHandler_1.createNameSpaceHandler(key, cacheConfig[key]));
    }
}
server_1.Server.on('request', async (req, res) => {
    const ctx = ctx_provider_1.CtxProvider(req, res);
    const fnMiddleware = compose(middlewares);
    res.statusCode = 404;
    const onerror = (err) => res.end(err);
    const handleResponse = () => ctx.respond();
    return fnMiddleware(ctx).then(handleResponse).catch(onerror);
});
